// Copyright (c) 2017, Marty Mills <daggerbot@gmail.com>
// This software is available under the terms of the zlib license.
// See COPYING.md for more information.

use std::marker::PhantomData;

use dcolor::{Color, Rgb};
use dimage::{self, ImageExt, VecImage2};
use dterm_built_in_font::{FONT, FONT_SIZE};
use dvec::Vec2;
use try_from::TryInto;

use cp437;

/// Built-in font loader.
pub struct Loader<C> {
    _phantom: PhantomData<C>,
    next: Option<u8>,
}

impl<C> Loader<C> {
    pub fn new () -> Loader<C> {
        Loader {
            _phantom: PhantomData,
            next: Some(0),
        }
    }
}

impl<C> Iterator for Loader<C>
    where cp437::Char: TryInto<C>
{
    type Item = (C, VecImage2<Rgb<u8>>);

    fn next (&mut self) -> Option<(C, VecImage2<Rgb<u8>>)> {
        loop {
            let byte = match self.next {
                None => { return None; },
                Some(n) => {
                    self.next = n.checked_add(1);
                    n
                },
            };

            let ch = match cp437::Char::from(byte).try_into() {
                Ok(ch) => ch,
                Err(_) => { continue; },
            };

            let mut glyph = dimage::fill(Rgb::black()).with_size2(FONT_SIZE).render_vec2();

            for y in 0..FONT_SIZE.y {
                for x in 0..FONT_SIZE.x {
                    if FONT[byte as usize][y as usize] & (1 << (FONT_SIZE.x - x - 1)) != 0 {
                        glyph.set_pixel(Vec2::new(x, y), Rgb::white());
                    }
                }
            }

            return Some((ch, glyph));
        }
    }
}
