// Copyright (c) 2017, Marty Mills <daggerbot@gmail.com>
// This software is available under the terms of the zlib license.
// See COPYING.md for more information.

use std;
use std::fmt::{self, Display, Formatter};

use dwindow;

/// `dterm` result type.
pub type Result<T> = std::result::Result<T, Error>;

/// `dterm` error type.
#[derive(Clone, Debug)]
pub enum Error {
    DWindow(dwindow::Error),
    DuplicateChar,
    InvalidGlyphSize,
    MissingOpenglFunction(String),
    NoChars,
    NoFallbackGlyph,
    NoFont,
    OpenglError(u32),
    UnsupportedOpenglVersion,
}

impl Display for Error {
    fn fmt (&self, f: &mut Formatter) -> fmt::Result {
        f.write_str(std::error::Error::description(self))?;

        match *self {
            Error::MissingOpenglFunction(ref name) => { write!(f, " ({})", name)?; },
            Error::OpenglError(n) => { write!(f, " ({})", n)?; },
            _ => ()
        }

        Ok(())
    }
}

impl std::error::Error for Error {
    fn cause (&self) -> Option<&std::error::Error> {
        match *self {
            Error::DWindow(ref inner) => Some(inner),
            _ => None
        }
    }

    fn description (&self) -> &str {
        match *self {
            Error::DWindow(ref inner) => inner.description(),
            Error::DuplicateChar => "duplicate char",
            Error::InvalidGlyphSize => "invalid glyph size",
            Error::MissingOpenglFunction(_) => "missing OpenGL function",
            Error::NoChars => "no chars",
            Error::NoFallbackGlyph => "no fallback glyph",
            Error::NoFont => "no font",
            Error::OpenglError(_) => "OpenGL error",
            Error::UnsupportedOpenglVersion => "unsupport OpenGL version",
        }
    }
}

impl From<dwindow::Error> for Error {
    fn from (inner: dwindow::Error) -> Error { Error::DWindow(inner) }
}
