// Copyright (c) 2017, Marty Mills <daggerbot@gmail.com>
// This software is available under the terms of the zlib license.
// See COPYING.md for more information.

#[cfg(feature = "gl")]
mod gl21;

#[cfg(feature = "gl")]
pub use self::gl21::Renderer;

use std::rc::Rc;

use dcolor::Rgb;
use dvec::{Rect2, Vec2};
use dwindow::{Display, Visual, Window};

use ::Coord;
use error::Result;
use font::Font;
use terminal::TerminalBuilder;

/// Renderer trait.
pub trait Render<C> : Sized {
    type Visual : Sized + AsRef<Visual>;

    fn choose_visual (display: &Display) -> Result<Self::Visual>;
    fn clear (&mut self, color: Rgb<u8>);
    fn damage (&mut self, rect: Rect2<Coord>);
    fn init (builder: &TerminalBuilder<C>, window: &Window, visual: Self::Visual) -> Result<Self>;
    fn resize_buf (&mut self, size: Vec2<Coord>);
    fn set_cell (&mut self, pt: Vec2<Coord>, ch: C, fg: Rgb<u8>, bg: Rgb<u8>);
    fn set_font (&mut self, window: &Window, font: &Rc<Font<C>>) -> Result<()>;
    fn set_scale (&mut self, scale: Vec2<Coord>);
    fn update (&mut self, window: &Window) -> Result<()>;
}
